package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.enums.OuLevelDic;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_organization_rel.GetSysOrganizationRelInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.XPageInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static pps.core.system.error.OrganizationError.PARAM_EMPTY;

@XText("组织机构接口")
@XService
public class SysOrganizationCloudServiceImpl implements SysOrganizationCloudService {


    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getSysOrganizationById(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getId());
        SysOrganizationEnt ent = mapper.selectOne(queryWrapper);
        GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getSysOrganizationByIds(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        List<SysOrganizationEnt> list = mapper.selectBatchIds(input.getIds());
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);

        return XListResult.success(outputs);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getSysOrganizationByNames(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<SysOrganizationEnt> lambda = queryWrapper.lambda();
        lambda.in(SysOrganizationEnt::getOuName, input.getIds());
        XPageInput pageInput = new XPageInput();
        pageInput.setPage(1);
        pageInput.setLimit(1000);
        XPageResult<GetSysOrganizationViewOutput> query = XMapperHelper.query(mapper, pageInput, queryWrapper, GetSysOrganizationViewOutput.class);
        return XListResult.success(query.getResult().getItems());
    }

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getSysOrganizationByOuLevelAndOuId(XContext context, GetSysOrganizationViewInput input) {

        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        XCopyUtils.copyObject(input, view);
        view.setEndTime(new Date());
        try {
            view.setUserId(context.getSession(PpsUserSession.class).getId());
        } catch (Exception e) {
            view.setUserId("");
        }
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
        GetSysOrganizationViewOutput output = XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getSysOrganizationByOuLevelAndUserId(XContext context, GetSysOrganizationViewInput input) {

        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        XCopyUtils.copyObject(input, view);
        view.setEndTime(new Date());
        view.setUserId(context.getSession(PpsUserSession.class).getId());
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndUserId(view);
        GetSysOrganizationViewOutput output = XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
        return XSingleResult.success(output);
    }


    @Override
    public XListResult<GetSysOrganizationViewOutput> getSysOrganizationListByOuLevelAndUserId(XContext context, GetSysOrganizationViewInput input) {

        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        XCopyUtils.copyObject(input, view);
        view.setEndTime(new Date());
        view.setUserId(input.getUserId());
        List<SysOrganizationView> sysOrganizationView = viewMapper.getSysOrganizationListByOuLevelAndUserId(view);
        List<GetSysOrganizationViewOutput> output = new ArrayList<>();
        for (SysOrganizationView item : sysOrganizationView) {
            GetSysOrganizationViewOutput itemOutput = XCopyUtils.copyNewObject(item, GetSysOrganizationViewOutput.class);
            output.add(itemOutput);
        }
        return XListResult.success(output);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getSysOrganizationByUserId(XContext context, GetSysOrganizationViewInput input) {
        SysUserOrganizationRelViewMapper userOrganizationRelViewMapper = context.getBean(SysUserOrganizationRelViewMapper.class);
        SysUserOrganizationRelView record = new SysUserOrganizationRelView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        List<SysUserOrganizationRelView> list = userOrganizationRelViewMapper.getSysOrganizationByUserId(record);
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getOuByUserId(XContext context, GetSysOrganizationViewInput input) {
        SysUserOrganizationRelMapper mapper = context.getBean(SysUserOrganizationRelMapper.class);
        SysOrganizationMapper sysOrganizationMapper = context.getBean(SysOrganizationMapper.class);
        SysOrganizationRelMapper sysOrganizationRelMapper = context.getBean(SysOrganizationRelMapper.class);


        QueryWrapper<SysUserOrganizationRelEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysUserOrganizationRelEnt::getUserId, input.getUserId())
                .le(SysUserOrganizationRelEnt::getEffectTime, new Date())
                .ge(SysUserOrganizationRelEnt::getEndTime, new Date());
        List<SysUserOrganizationRelEnt> relEnts = mapper.selectList(wrapper);
        GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
        if (!CollectionUtils.isEmpty(relEnts)) {
            if (XStringUtils.isNotEmpty(relEnts.get(0).getOuId())) {
                QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysOrganizationEnt::getId, relEnts.get(0).getOuId())
                        .eq(SysOrganizationEnt::getIsDeleted, 1);
                SysOrganizationEnt sysOrganizationEnt = sysOrganizationMapper.selectOne(queryWrapper);
                XCopyUtils.copyObject(sysOrganizationEnt, output);
                output.setOuId(relEnts.get(0).getOuId());

                QueryWrapper<SysOrganizationRelEnt> qw = new QueryWrapper<>();
                qw.lambda().eq(SysOrganizationRelEnt::getOuId, relEnts.get(0).getOuId())
                        .le(SysOrganizationRelEnt::getEffectTime, new Date())
                        .ge(SysOrganizationRelEnt::getEndTime, new Date());
                List<SysOrganizationRelEnt> sysOrganizationRelEnts = sysOrganizationRelMapper.selectList(qw);
                if (!CollectionUtils.isEmpty(sysOrganizationRelEnts)) {
                    output.setOuPath(sysOrganizationRelEnts.get(0).getOuIdPath());
                }
            }
        }
        return XSingleResult.success(output);
    }

    @Override
    public XPageResult<GetSysOrganizationViewOutput> getSysOrganizationList(XContext context, QuerySysOrganizationViewInput input) {
        SysUserOrganizationRelViewMapper userOrganizationRelViewMapper = context.getBean(SysUserOrganizationRelViewMapper.class);
        SysUserOrganizationRelView record = new SysUserOrganizationRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(1, 10000);
        List<SysOrganizationView> list = userOrganizationRelViewMapper.selectOrgList(record);
        PageInfo<SysOrganizationView> pageInfo = new PageInfo<>(list);
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getOrganizationByParentOuId(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationViewMapper userOrganizationRelViewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        record.setIsDeleted(1);
        XCopyUtils.copyObject(input, record);
        List<SysOrganizationView> list = userOrganizationRelViewMapper.selectOrganizationByParentOuId(record);
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getOrganizationBySubOuId(XContext context, GetSysOrganizationViewInput input) {
        if (input.getId() == null) {
            return XSingleResult.error(context, PARAM_EMPTY);
        }
        SysOrganizationRelMapper mapper = context.getBean(SysOrganizationRelMapper.class);
        QueryWrapper<SysOrganizationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getId());
        queryWrapper.lambda().lt(SysOrganizationRelEnt::getEffectTime, new Date());
        queryWrapper.lambda().gt(SysOrganizationRelEnt::getEndTime, new Date());
        SysOrganizationRelEnt relEnt = mapper.selectOne(queryWrapper);

        SysOrganizationMapper orgMapper = context.getBean(SysOrganizationMapper.class);
        SysOrganizationEnt sysOrganizationEnt = orgMapper.selectById(relEnt.getParentOuId());

        GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
        XCopyUtils.copyObject(sysOrganizationEnt, output);
        return XSingleResult.success(output);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getOuIdsByUserId(XContext context, GetSysOrganizationViewInput input) {
        SysAuthOuScopeViewMapper scopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
        List<SysAuthOuScopeView> sysAuthOuScopeViews = scopeViewMapper.selectOuIdsByUserId(input.getUserId());
        List<GetSysOrganizationViewOutput> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sysAuthOuScopeViews)) {
            sysAuthOuScopeViews.forEach(sysAuthOuScopeView -> {
                GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
                output.setOuId(sysAuthOuScopeView.getOuId());
                output.setOuName(sysAuthOuScopeView.getOuName());
                list.add(output);
            });
        }
        return XListResult.success(list);
    }

    @Override
    @XText("获取授权组织机构id")
    public XListResult<GetSysOrganizationViewOutput> getOuIdsByCondition(XContext context, GetOuIdsByConditionInput input) {
        SysAuthOuScopeViewMapper scopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
        SysAuthOuScopeView record = XCopyUtils.copyNewObject(input, SysAuthOuScopeView.class);
        List<SysAuthOuScopeView> sysAuthOuScopeViews = scopeViewMapper.selectOuInfoByCondition(record);
        List<GetSysOrganizationViewOutput> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sysAuthOuScopeViews)) {
            sysAuthOuScopeViews.forEach(sysAuthOuScopeView -> {
                GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
                output.setOuId(sysAuthOuScopeView.getOuId());
                output.setOuName(sysAuthOuScopeView.getOuName());
                list.add(output);
            });
        }
        return XListResult.success(list);
    }


    @XText("根据用户id获取用户组织机构")
    public XListResult<OuIdAndOutNameOutput> getOrgListByUserId(XContext context, QueryUserIdInput input) {
        SysUserOrganizationRelViewMapper mapper = context.getBean(SysUserOrganizationRelViewMapper.class);
        List<SysUserOrganizationRelView> list = mapper.getOrgListByUserId(input.getUserId());
        List<OuIdAndOutNameOutput> outputs = XCopyUtils.copyNewList(list, OuIdAndOutNameOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据组织机构id获取所有的子级组织机构")
    public XListResult<GetOuListTreeOutput> getAllOuListByOuId(XContext context, GetAllOuListByOuIdInput input) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(input.getOuId());
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return XListResult.success(outputs);
    }

    @XText("根据组织机构id获取所有的子级组织机构")
    public XListResult<GetOuListTreeOutput> getOuListByOuIds(XContext context, GetOuListByOuIdsInput input) {
        if (CollectionUtils.isEmpty(input.getOuIdList())) return XListResult.success(new ArrayList<>());
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> list = mapper.selectListByOuIds(input.getOuIdList());
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<GetOuIdPathByOuIdOutput> getOuIdPathByOuId(XContext context, GetOuIdPathByOuIdInput input) {
        SysOrganizationRelMapper mapper = context.getBean(SysOrganizationRelMapper.class);
        QueryWrapper<SysOrganizationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getOuId());
        queryWrapper.lambda().gt(SysOrganizationRelEnt::getEndTime, new Date());// 结束时间
        queryWrapper.lambda().lt(SysOrganizationRelEnt::getEffectTime, new Date()); //有效时间
        queryWrapper.lambda().orderByDesc(SysOrganizationRelEnt::getCreateTime);
        queryWrapper.lambda().last(" limit 1");
        SysOrganizationRelEnt relEnt = mapper.selectOne(queryWrapper);
        GetOuIdPathByOuIdOutput output = XCopyUtils.copyNewObject(relEnt, GetOuIdPathByOuIdOutput.class);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetSysOrganizationIdPathOutput> getOrganizationPath(XContext context) {
        GetSysOrganizationViewInput orgInput = new GetSysOrganizationViewInput();
        GetSysOrganizationIdPathOutput output = new GetSysOrganizationIdPathOutput();
        output.setStationOuId(context.getSession(PpsUserSession.class).getOuId());
        orgInput.setOuLevel(OuLevelDic.OuLevelUnit.getKey());
        XSingleResult<GetSysOrganizationViewOutput> unitResult = getSysOrganizationByOuLevelAndUserId(context, orgInput);
        output.setUnitOuId(unitResult.getResult().getId());
        orgInput.setOuLevel(OuLevelDic.OuLevelArea.getKey());
        XSingleResult<GetSysOrganizationViewOutput> companyResult = getSysOrganizationByOuLevelAndUserId(context, orgInput);
        output.setCompanyOuId(companyResult.getResult().getId());
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetOrganizationOuLevel> getOrganizationOuLevelOuId(XContext context) {
        String ouId = context.getSession(PpsUserSession.class).getOuId();
        String ouLevel = context.getSession(PpsUserSession.class).getOuLevel();
        GetOrganizationOuLevel orgOuLevel = getOrgOuLevel(context, ouId, ouLevel);
        return XSingleResult.success(orgOuLevel);
    }

    private GetOrganizationOuLevel getOrgOuLevel(XContext context, String ouId, String ouLevel) {
        OuLevelDic byKey = OuLevelDic.getByKey(ouLevel);
        if (byKey == null) {
            GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
            input.setId(ouId);
            XSingleResult<GetSysOrganizationViewOutput> organizationBySubOuId = getOrganizationBySubOuId(context, input);
            getOrgOuLevel(context, organizationBySubOuId.getResult().getId(), organizationBySubOuId.getResult().getOuLevel());
        }
        GetOrganizationOuLevel out = new GetOrganizationOuLevel();
        out.setOuId(ouId);
        out.setOuLevel(ouLevel);
        return out;
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getOrganizationBySubOuIds(XContext context, GetSysOrganizationViewInput input) {
        if (CollectionUtils.isEmpty(input.getIds())) {
            return XListResult.error(context, PARAM_EMPTY);
        }
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> views = mapper.selectOrganizationBySubOuIds(input.getIds());
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(views, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> selectRegionFirmByUserId(XContext context, SelectRegionFirmInput input) {
        SysOrganizationService bean = context.getBean(SysOrganizationService.class);
        XListResult<pps.core.system.service.data.sys_organization.GetSysOrganizationViewOutput> listResult = bean.selectRegionFirmByUserId(context, input);
        if (CollectionUtils.isEmpty(listResult.getResult())) {
            return XListResult.success(new ArrayList<>());
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(listResult.getResult(), GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> queryOuInfoOrStationByParentOuId(XContext context) {
        PpsUserSession session = context.getSession(PpsUserSession.class);

        SysOrganizationService organizationService = context.getBean(SysOrganizationService.class);
        GetSysOrganizationRelInput organizationRelInput = new GetSysOrganizationRelInput();
        organizationRelInput.setParentOuId(session.getOuId());
        organizationRelInput.setOuLevel(session.getOuLevel());
        XListResult<pps.core.system.service.data.sys_organization.GetSysOrganizationViewOutput> listResult = organizationService.selectOuInfoOrStationByParentOuId(context, organizationRelInput);
        List<GetSysOrganizationViewOutput> outList = XCopyUtils.copyNewList(listResult.getResult(), GetSysOrganizationViewOutput.class);
        return XListResult.success(outList);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> selectOrganizationAllByOrganizationName(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        record.setOuName(input.getOuName());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        List<SysOrganizationView> list = mapper.selectOrganizationEffectiveByOuName(record);
        List<GetSysOrganizationViewOutput> viewList = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XListResult.success(viewList);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getAllListByOuId(XContext context, GetAllOuListByOuIdInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(input.getOuId());
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        List<GetSysOrganizationViewOutput> viewList = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XListResult.success(viewList);
    }
}
