package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.core.base.entity.BasePhotovoltaicPlantEnt;
import pps.core.base.mapper.BasePhotovoltaicPlantMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.List;

/**
 * 光伏电站Cloud模块
 *
 * @author TianChao
 * @date 2023/09/27
 */
@XService
public class BasePhotovoltaicPlantCloudServiceImpl implements BasePhotovoltaicPlantCloudService {

    /**
     * 光伏电站Cloud模块-获取电站列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    @Override
    public XListResult<GetBasePhotovoltaicPlantCloudOutput> getBasePhotovoltaicPlantList(XContext context, GetBasePhotovoltaicPlantCloudInput input) {
        List<String> plantIds = input.getPlantIds();
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        List<BasePhotovoltaicPlantEnt> list = mapper.selectList(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(CollUtil.isNotEmpty(plantIds), BaseModel::getId, plantIds)
        );
        List<GetBasePhotovoltaicPlantCloudOutput> outPuts = XCopyUtils.copyNewList(list, GetBasePhotovoltaicPlantCloudOutput.class);
        return XListResult.success(outPuts);
    }
}
