package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.GetSysAreaInput;
import pps.cloud.system.service.data.GetSysAreaOutput;
import pps.core.base.entity.BasePhotovoltaicPlantEnt;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.entity.BasePowerLinePlantEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BasePhotovoltaicPlantMapper;
import pps.core.base.mapper.BasePowerLinePlantMapper;
import pps.core.base.service.data.base_photovoltaic_plant.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.constant.UserConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.SysOrganizationView;
import pps.core.system.mapper.SysOrganizationViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;
import java.util.Optional;


/**
 * 光伏电站模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BasePhotovoltaicPlantService {

    /**
     * 光伏电站模块--新增
     * POST /base/base-photovoltaic-plant/create-base-photovoltaic-plant
     * 接口ID：105466278
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466278
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBasePhotovoltaicPlant(XContext context, CreateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (StringUtils.isNotEmpty(input.getMakerNumber()) && this.checkMakerNumberExists(mapper, input.getMakerNumber(), null)) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BasePhotovoltaicPlantEnt entity = XCopyUtils.copyNewObject(input, BasePhotovoltaicPlantEnt.class);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--修改
     * POST /base/base-photovoltaic-plant/update-base-photovoltaic-plant
     * 接口ID：105466279
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466279
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--修改")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePhotovoltaicPlant(XContext context, UpdateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (StringUtils.isNotEmpty(input.getMakerNumber()) && this.checkMakerNumberExists(mapper, input.getMakerNumber(), input.getId())) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--删除
     * POST /base/base-photovoltaic-plant/delete-base-photovoltaic-plant
     * 接口ID：105466280
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466280
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePhotovoltaicPlant(XContext context, DeleteBasePhotovoltaicPlantInput input) {
        String plantId = input.getId();
        if (this.checkPhotovoltaicIsReference(context, plantId)) {
            return XServiceResult.error(context, BusinessError.PhotovoltaicReference);
        }
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (Objects.isNull(this.selectOneByPlantId(plantId, mapper))) {
            return XServiceResult.error(context, XError.NotFound);
        }
        BasePhotovoltaicPlantEnt entity = new BasePhotovoltaicPlantEnt();
        entity.setId(plantId);
        entity.setIsDeleted(BusinessConstant.ZERO);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--详情
     * GET /base/base-photovoltaic-plant/get-base-photovoltaic-plant
     * 接口ID：105466276
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466276
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePhotovoltaicPlantOutput}>
     */
    @XText("光伏电站模块--详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePhotovoltaicPlantOutput> getBasePhotovoltaicPlant(XContext context, GetBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePhotovoltaicPlantOutput output = XCopyUtils.copyNewObject(entity, GetBasePhotovoltaicPlantOutput.class);
        //根据组织机构的路径获取
        SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView organizationView = new SysOrganizationView();
        organizationView.setId(entity.getOuId());
        organizationView = organizationViewMapper.selectOrgProvince(organizationView);
        if (Objects.nonNull(organizationView)) {
            output.setProvince(organizationView.getProvinceName());
        }
        output.setAreaName(Optional.ofNullable(this.getSysAreaInfo(context, GetSysAreaInput.builder()
                        .id(entity.getAreaCode())
                        .build()))
                .map(GetSysAreaOutput::getName)
                .orElse(null));
        return XSingleResult.success(output);
    }

    /**
     * 光伏电站模块--分页查询
     * GET /base/base-photovoltaic-plant/query-base-photovoltaic-plant
     * 接口ID：105466277
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466277
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePhotovoltaicPlantOutput}>
     */
    @XText("光伏电站模块--分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePhotovoltaicPlantOutput> queryBasePhotovoltaicPlant(XContext context, QueryBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        String stationName = input.getStationName();
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists(!StringUtils.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId()), "SELECT 1 FROM base_power_line_plant w WHERE base_photovoltaic_plant.id = w.plant_id AND w.is_deleted = 1 AND w.line_id <> {0}", input.getLineId())
                    .notExists(StringUtils.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId()), "SELECT 1 FROM base_power_line_plant w WHERE base_photovoltaic_plant.id = w.plant_id AND w.is_deleted = 1")
                    .eq(BasePhotovoltaicPlantEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BasePhotovoltaicPlantEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .like(CharSequenceUtil.isNotEmpty(stationName), BasePhotovoltaicPlantEnt::getStationName, stationName)
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE)
                .orderByDesc(BasePhotovoltaicPlantEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePhotovoltaicPlantOutput.class);
    }

    @XText("根据父级id查询儿子级(含电站)")
    @XApiGet
    public XListResult<GetSysOrganizationAndPlantViewOutput> selectOuAndPhotovoltaicPlantInfoByParentOuId(XContext context, GetSysOrganizationAndPlantInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        BasePhotovoltaicPlantMapper plantMapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysOrganizationView> viewList;
        String ouId = "00000000-0000-0000-0000-000000000000";
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin)) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                ouId = session.getOuId();
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            ouId = input.getParentOuId();
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        queryWrapper.lambda().eq(BasePhotovoltaicPlantEnt::getOuId, ouId).eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE);
        List<BasePhotovoltaicPlantEnt> plantList = plantMapper.selectList(queryWrapper);
        plantList.forEach(item -> {
            SysOrganizationView view = new SysOrganizationView();
            view.setId(item.getId());
            view.setOuName(item.getStationName());
            view.setShortName(item.getStationName());
            view.setOuLevel("PHOTOVOLTAIC");
            viewList.add(view);
        });
        List<GetSysOrganizationAndPlantViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationAndPlantViewOutput.class);
        return XListResult.success(outputs);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查光伏电站是否被引用
     *
     * @param context 上下文
     * @param plantId 电站ID
     * @return boolean
     */
    private boolean checkPhotovoltaicIsReference(XContext context, String plantId) {
        BasePowerLinePlantMapper mapper = context.getBean(BasePowerLinePlantMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                .eq(BasePowerLinePlantEnt::getPlantId, plantId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过电站ID获取电站信息
     *
     * @param plantId 电站id
     * @param mapper  映射器
     * @return {@link BasePowerLineEnt}
     */
    private BasePhotovoltaicPlantEnt selectOneByPlantId(String plantId, BasePhotovoltaicPlantMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getId, plantId));
    }

    /**
     * 获取系统区域信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link GetSysAreaOutput}
     */
    private GetSysAreaOutput getSysAreaInfo(XContext context, GetSysAreaInput input) {
        SystemAreaService systemAreaService = context.getBean(SystemAreaService.class);
        XSingleResult<GetSysAreaOutput> singleResult = systemAreaService.getSysAreaByParam(context, input);
        singleResult.throwIfFail();
        return singleResult.getResult();
    }

    /**
     * 检查编号是否存在
     *
     * @param mapper      映射器
     * @param makerNumber 出场编号
     * @param id          id
     * @return boolean
     */
    private boolean checkMakerNumberExists(BasePhotovoltaicPlantMapper mapper, String makerNumber, String id) {
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BasePhotovoltaicPlantEnt::getMakerNumber, makerNumber)
                .ne(StringUtils.isNotEmpty(id), BaseModel::getId, id)
        );
        return count > 0;
    }
}
