package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.mapper.BasePriceStrategyDetailMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 市电峰谷配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/07 14:52
 */
@XService
public class BasePriceStrategyCloudServiceImpl implements IBasePriceStrategyCloudService {

    /**
     * 市电峰谷配置Cloud模块--策略配置详情列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetBasePriceStrategyDetailOutput}>
     */
    @Override
    public XListResult<GetBasePriceStrategyDetailOutput> queryStrategyDetailList(XContext context, GetBasePriceStrategyDetailInput input) {
        //查明细
        BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
        List<BasePriceStrategyDetailEnt> detailList = detailMapper.selectList(
                new LambdaQueryWrapper<BasePriceStrategyDetailEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(BasePriceStrategyDetailEnt::getStrategyMonth, input.getStrategyMonth())
                        .apply("strategy_id = ( SELECT strategy_id FROM base_power_line WHERE id = {0} ) ", input.getLineId())
                        .orderByAsc(BasePriceStrategyDetailEnt::getStartTime)
        );
        List<GetBasePriceStrategyDetailOutput> outputs;
        if (CollUtil.isEmpty(detailList)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(detailList, GetBasePriceStrategyDetailOutput.class);
        }
        return XListResult.success(outputs);
    }
}