package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import org.springframework.util.CollectionUtils;
import pps.core.base.entity.BaseLineAttenuationRateEnt;
import pps.core.base.entity.BaseLineAttenuationRateView;
import pps.core.base.entity.BaseProjectInfoEnt;
import pps.core.base.entity.BaseProjectInfoPredictedDataEnt;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_line_attenuation_rate.CreateBaseLineAttenuationRateInput;
import pps.core.base.service.data.base_project_info.*;
import pps.core.base.utils.HttpUtils;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysAreaEnt;
import pps.core.system.mapper.SysAreaMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@XService
public class BaseProjectInfoService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createProjectInfo(XContext context, CreateBaseProjectInfoInput input){
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = new BaseProjectInfoEnt();
        XCopyUtils.copyObject(input, entity);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setIsDeleted(0);
        mapper.insert(entity);
        List<BaseLineAttenuationRateView> list = new ArrayList<>();
        if(!CollectionUtils.isEmpty(input.getList())){
            BaseLineAttenuationRateViewMapper rateMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            for(CreateBaseLineAttenuationRateInput rateInput : input.getList()){
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput , BaseLineAttenuationRateView.class);
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                ent.setCreateTime(now);
                ent.setType(1);
                ent.setLineId(entity.getId());
                list.add(ent);
            }
            rateMapper.batchInsert(list);
        }
        //开始预测
        XThreadHelper.async(new Runnable() {
            @Override
            public void run() {
                predictedData(context , entity , list);
            }
        });
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateProjectInfo(XContext context, UpdateBaseProjectInfoInput input){
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        QueryWrapper<BaseProjectInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseProjectInfoEnt::getId, input.getId());
        BaseProjectInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
        entity.setIsDeleted(0);
        mapper.updateById(entity);
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        QueryWrapper<BaseLineAttenuationRateEnt> queryRateWrapper = new QueryWrapper<>();
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getLineId , entity.getId());
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getType , 1);
        rateMapper.delete(queryRateWrapper);
        List<BaseLineAttenuationRateView> list = new ArrayList<>();
        if(!CollectionUtils.isEmpty(input.getList())){
            BaseLineAttenuationRateViewMapper rateViewMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            for(CreateBaseLineAttenuationRateInput rateInput : input.getList()){
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput , BaseLineAttenuationRateView.class);
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                ent.setCreateTime(now);
                ent.setLineId(entity.getId());
                ent.setType(1);
                list.add(ent);
            }
            rateViewMapper.batchInsert(list);
        }
        //开始预测
        XThreadHelper.async(new Runnable() {
            @Override
            public void run() {
                predictedData(context , entity , list);
            }
        });
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteProjectInfo(XContext context, DeleteProjectInfoInput input){
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        QueryWrapper<BaseProjectInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseProjectInfoEnt::getId, input.getId());
        BaseProjectInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity.setIsDeleted(1);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseProjectInfoOutput> getProjectInfo(XContext context, GetProjectInfoInput input){
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        QueryWrapper<BaseProjectInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseProjectInfoEnt::getId, input.getId());
        BaseProjectInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseProjectInfoOutput output = new GetBaseProjectInfoOutput();
        XCopyUtils.copyObject(entity, output);
        if(XStringUtils.isNotEmpty(entity.getProvince()))
            output.setProvince(Integer.valueOf(entity.getProvince()));
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        QueryWrapper<BaseLineAttenuationRateEnt> queryRateWrapper = new QueryWrapper<>();
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getLineId , entity.getId());
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getType , 1);
        List<BaseLineAttenuationRateEnt> list = rateMapper.selectList(queryRateWrapper);
        if(!CollectionUtils.isEmpty(list)){
            List<CreateBaseLineAttenuationRateInput> rateInputs = XCopyUtils.copyNewList(list , CreateBaseLineAttenuationRateInput.class);
            output.setList(rateInputs);
        }
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseProjectInfoOutput> queryProjectInfoList(XContext context, QueryProjectInfoInput input){
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        QueryWrapper<BaseProjectInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseProjectInfoEnt::getIsDeleted , 0);
        if(XStringUtils.isNotEmpty(input.getProjectName()))
            queryWrapper.lambda().like(BaseProjectInfoEnt::getProjectName , input.getProjectName());
        if(input.getProductionYear() != null)
            queryWrapper.lambda().like(BaseProjectInfoEnt::getProductionYear , input.getProductionYear());
        List<BaseProjectInfoEnt> list = mapper.selectList(queryWrapper);
        PageInfo<BaseProjectInfoEnt> pageInfo = new PageInfo(list);
        List<QueryBaseProjectInfoOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseProjectInfoOutput.class);
        SysAreaMapper areaMapper = context.getBean(SysAreaMapper.class);
        List<SysAreaEnt> areaEntList = areaMapper.selectList(new QueryWrapper<>());
        outputs.stream().forEach(item->{
            if(XStringUtils.isNotEmpty(item.getProvince())){
                SysAreaEnt areaEnt = areaEntList.stream().filter(area->item.getProvince().equals(area.getId()+"")).findFirst().orElse(null);
                if(areaEnt != null)
                    item.setProvince(getParentNameStr(areaEntList ,areaEnt ));
            }
        });
        return  XPageResult.success(outputs, input, pageInfo.getTotal());
    }


    /**
     * 根据子节点获取最上层节点
     * @return
     */
    public static String getParentNameStr(List<SysAreaEnt> areaList, SysAreaEnt child){
        String parentNameStr = child.getName();
        Integer parentId = child.getParentId();
        if(parentId == 120){

        }else {
            SysAreaEnt parent = areaList.stream().filter(item -> item.getId().equals(parentId)).findFirst().orElse(null);
            String nameStr = getParentNameStr(areaList, parent);
            parentNameStr = nameStr +"/" + parentNameStr;
        }
        return parentNameStr;
    }

    @XApiAnonymous
    @XApiGet
    public XListResult<GetProjectInfoPredictedDataOutput> queryProjectInfoPredictedDataList(XContext context, GetProjectInfoPredictedDataInput input){
        BaseProjectInfoPredictedDataMapper mapper = context.getBean(BaseProjectInfoPredictedDataMapper.class);
        QueryWrapper<BaseProjectInfoPredictedDataEnt> queryWrapper = new QueryWrapper<>();
        if(input.getProjectId() != null)
            queryWrapper.lambda().eq(BaseProjectInfoPredictedDataEnt::getProjectId , input.getProjectId());
        List<BaseProjectInfoPredictedDataEnt> list = mapper.selectList(queryWrapper);
        PageInfo<BaseProjectInfoPredictedDataEnt> pageInfo = new PageInfo(list);
        List<GetProjectInfoPredictedDataOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), GetProjectInfoPredictedDataOutput.class);
        return  XListResult.success(outputs);
    }

    private void predictedData(XContext context , BaseProjectInfoEnt entity , List<BaseLineAttenuationRateView> list){
        CalcProjectInfoPredictedDataInput input = new CalcProjectInfoPredictedDataInput();
        input.setLat(entity.getLatitude());
        input.setLon(entity.getLongitude());
        input.setAngle(entity.getArrayIncidence());
        input.setDirection(entity.getArrayToward());
        input.setCapacity(entity.getInstallCapacity());
        input.setEfficiency(new BigDecimal(0.35).setScale(2 , RoundingMode.HALF_EVEN));
        input.setWeather(new BigDecimal(0.7).setScale(2, RoundingMode.HALF_EVEN));
        List<BaseLineAttenuationRateView> sortList = list.stream().sorted(Comparator.comparing(BaseLineAttenuationRateView::getSort)).collect(Collectors.toList());
        List<BigDecimal> decimals = sortList.stream().map(BaseLineAttenuationRateView::getAttenuationRate).collect(Collectors.toList());
        input.setDecline(decimals);
        Map<String , Object> map = getPredictedData(context , input);
        //插入数据
        BaseProjectInfoPredictedDataMapper mapper = context.getBean(BaseProjectInfoPredictedDataMapper.class);
        QueryWrapper<BaseProjectInfoPredictedDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseProjectInfoPredictedDataEnt::getProjectId , entity.getId());
        mapper.delete(queryWrapper);
        List<Double> powers = (List<Double>)map.get("power");
        Integer particularYear = Integer.valueOf(entity.getProductionYear());
        PpsUserSession userSession = context.getSession(PpsUserSession.class);
        Date date = new Date();
        List<BaseProjectInfoPredictedDataEnt> entList = new ArrayList<>();
        for(Double decimal : powers){
            BaseProjectInfoPredictedDataEnt ent = new BaseProjectInfoPredictedDataEnt();
            ent.setProjectId(entity.getId());
            ent.setParticularYear(particularYear + "");
            ent.setPredictedPower(BigDecimal.valueOf(decimal.doubleValue()));
            ent.setCreateById(userSession.getId());
            ent.setCreateByName(userSession.getUserName());
            ent.setCreateTime(date);
            particularYear ++;
            entList.add(ent);
        }
        BaseProjectInfoPredictedDataViewMapper dataViewMapper = context.getBean(BaseProjectInfoPredictedDataViewMapper.class);
        dataViewMapper.batchInsert(entList);
    }

    public Map<String , Object> getPredictedData(XContext context, CalcProjectInfoPredictedDataInput calcProjectInfoPredictedDataInput ){
        Map<String , Object> returnMap = new HashMap<>();
        try {
            String jsonString =  XJsonUtils.toJson(calcProjectInfoPredictedDataInput);
            context.getLogger().info(jsonString);
            Object ret = HttpUtils.send2( "http://121.36.43.217:8098//model/yearmodel",
                    jsonString);
            returnMap = XJsonUtils.toMap(ret.toString());
            return returnMap;
        }catch (Exception e) {
            throw new RuntimeException("调用预测接口失败");
        }
    }
}
