package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 井口配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 17:34
 */
@XService
public class BaseWellheadCloudServiceImpl implements IBaseWellheadCloudService {

    /**
     * 井口配置Cloud模块--条件查询井口基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBaseWellheadOutput}>
     */
    @Override
    public XListResult<DynamicQueryBaseWellheadOutput> queryBaseWellheadListByParam(XContext context, DynamicQueryBaseWellheadInput input) {
        String wellheadId = input.getWellheadId();
        List<String> wellheadIds = input.getWellheadIds();
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        List<BaseWellheadEnt> list = mapper.selectList(
                new LambdaQueryWrapper<BaseWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(StringUtils.isNotBlank(wellheadId), BaseModel::getId, wellheadId)
                        .in(CollUtil.isNotEmpty(wellheadIds), BaseModel::getId, wellheadIds)
        );
        List<DynamicQueryBaseWellheadOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(
                    list,
                    DynamicQueryBaseWellheadOutput.class
            );
        }
        return XListResult.success(outputs);
    }
}
