package pps.core.base.utils;

import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2023/08/25 16:08
 */
public class ServiceUtil {

    /*-------------------------服务调用-------------------------*/

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getDictMap(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult().stream()
                .collect(Collectors.toMap(QuerySysDictionaryViewOutput::getDicKey, QuerySysDictionaryViewOutput::getDicName));
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public static List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }
}