package pps.core.task.job;

import pps.cloud.space.service.ISpaceCalibrationCloudService;
import pps.core.task.constant.CronConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XLoggerTool;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 定期校准定时任务
 *
 * @author ZWT
 * @date 2023/09/20 09:20
 */
@XText("长期间开优化定时任务")
@XService
public class SpaceCalibrationJob implements XJob {

    /**
     * 每天00:15执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = CronConstant.EVERY_DAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        XLoggerTool logger = xContext.getLogger();
        logger.info("------ SpaceCalibrationJob start:{}", System.currentTimeMillis());
        ISpaceCalibrationCloudService service = xContext.getBean(ISpaceCalibrationCloudService.class);
        XServiceResult result = service.calibrateJob(xContext);
        logger.info("------ SpaceCalibrationJob end:{}", System.currentTimeMillis());
        result.throwIfFail();
        return XServiceResult.OK;
    }
}
