package pps.core.task.job;

import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.task.constant.CronConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XLoggerTool;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 天气数据邮件接收定时任务
 *
 * @author ZWT
 * @date 2023/11/13 17:15
 */
@XText("天气数据邮件接收定时任务")
@XService
public class WeatherReceiveJob implements XJob {

    /**
     * 半小时执行一次
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = CronConstant.HALF_HOUR)
    @Override
    public XServiceResult execute(XContext xContext) {
        XLoggerTool logger = xContext.getLogger();
        logger.info("------ WeatherReceiveJob start:{}", System.currentTimeMillis());
        IBaseWeatherCloudService service = xContext.getBean(IBaseWeatherCloudService.class);
        XServiceResult result = service.weatherDataProcess(xContext);
        result.throwIfFail();
        logger.info("------ WeatherReceiveJob end:{}", System.currentTimeMillis());
        return XServiceResult.OK;
    }
}
