package pps.core.space.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionWellheadEnt;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XSingleResult;

/**
 * 基础间开配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/06 17:09
 */
@XService
public class SpaceInstitutionDetailCloudServiceImpl implements ISpaceInstitutionDetailCloudService {

    /**
     * 基础间开配置Cloud模块--获取线路配置制度数量
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link XSingleResult}<{@link Long}>
     */
    @Override
    public XSingleResult<Long> getInstitutionLineCountByLineId(XContext context, String lineId) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return XSingleResult.success(mapper.selectCount(
                new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceInstitutionDetailEnt::getLineId, lineId)
        ));
    }

    /**
     * 基础间开配置Cloud模块--检查井口是否被引用
     *
     * @param context    上下文
     * @param wellheadId 井口id
     * @return {@link XSingleResult}<{@link Long}>
     */
    @Override
    public XSingleResult<Long> getInstitutionWellheadCountByWellheadId(XContext context, String wellheadId) {
        SpaceInstitutionWellheadMapper mapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        return XSingleResult.success(mapper.selectCount(
                new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceInstitutionWellheadEnt::getWellheadId, wellheadId)
        ));
    }
}
