package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("短期间开优化模块")
public class SpaceOptimizeShortPeriodService {

    /**
     * 短期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeShortPeriodViewOutput> getSpaceOptimizeShortPeriodView(XContext context, GetSpaceOptimizeShortPeriodViewInput input) {
        GetSpaceOptimizeShortPeriodViewOutput output = new GetSpaceOptimizeShortPeriodViewOutput();
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        //查询当前线路的基础制度
        DateTime date = DateUtil.date();
        SpaceInstitutionDetailEnt detailEnt = detailMapper.selectOne(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .eq(SpaceInstitutionDetailEnt::getLineId, input.getLineId())
                .le(SpaceInstitutionDetailEnt::getInstitutionStartDate, date)
                .ge(SpaceInstitutionDetailEnt::getInstitutionEndDate, date)
                .last("LIMIT 1")
        );
        if (Objects.isNull(detailEnt)) {
            //当前线路没有基础制度
            output.setWellheadList(new ArrayList<>(0));
        } else {
            output.setGridTypeKey(detailEnt.getGridTypeKey());
            //查出当前制度所有的井口
            String institutionId = detailEnt.getId();
            SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
            List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                    .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
            );
            if (CollUtil.isEmpty(wellheadEntList)) {
                //缺少井口
                output.setWellheadList(new ArrayList<>(0));
            }
            SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
            //查当日是否存在优化记录
            SpaceOptimizeShortPeriodEnt period = mapper.selectOne(
                    new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                            .eq(SpaceOptimizeShortPeriodEnt::getLineId, input.getLineId())
                            .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                            .ge(SpaceOptimizeShortPeriodEnt::getOptimizeDate, DateUtil.beginOfDay(DateUtil.date()))
            );
            Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(0);
            if (Objects.nonNull(period)) {
                //如果存在,查出优化过的区间
                SpaceOptimizeShortDurationViewMapper durationViewMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
                List<SpaceOptimizeShortDurationView> durationViewList = durationViewMapper.selectList(SpaceOptimizeShortDurationView.builder()
                        .shortPeriodId(period.getId())
                        .build()
                );
                if (CollUtil.isNotEmpty(durationViewList)) {
                    collect = XCopyUtils.copyNewList(durationViewList, SpaceOptimizeDateDuration.class)
                            .stream()
                            .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
                }
            } else {
                //如果没有,查初始区间
                SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
                List<SpaceInstitutionDurationEnt> durationEntList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                        .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
                );
                if (CollUtil.isNotEmpty(durationEntList)) {
                    collect = XCopyUtils.copyNewList(durationEntList, SpaceOptimizeDateDuration.class)
                            .stream()
                            .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
                }
            }
            //遍历井口,添加区间
            List<GetSpaceOptimizeShortWellheadViewOutput> wellheadViewOutputs = XCopyUtils.copyNewList(wellheadEntList, GetSpaceOptimizeShortWellheadViewOutput.class);
            for (GetSpaceOptimizeShortWellheadViewOutput wellheadViewOutput : wellheadViewOutputs) {
                if (collect.containsKey(wellheadViewOutput.getWellheadId())) {
                    wellheadViewOutput.setDurationList(ServiceUtil.getAllDateDuration(collect.get(wellheadViewOutput.getWellheadId())));
                } else {
                    //大间开处理
                    if (StringUtils.equals(BusinessConstant.STRING_ONE, wellheadViewOutput.getIntervalTypeKey())) {
                        Integer isOpen = BusinessConstant.ZERO;
                        String closeWellTime = BusinessConstant.START_OF_DAY_TIME;
                        if (ServiceUtil.checkWellIsOpen(wellheadViewOutput.getOpenWellDay(), wellheadViewOutput.getCloseWellDay(), wellheadViewOutput.getCreateTime(), date)) {
                            isOpen = BusinessConstant.ONE;
                            closeWellTime = BusinessConstant.END_OF_DAY_TIME;
                        }
                        wellheadViewOutput.setDurationList(
                                CollUtil.newArrayList(SpaceOptimizeDateDuration.builder()
                                        .openWellTime(BusinessConstant.START_OF_DAY_TIME)
                                        .closeWellTime(closeWellTime)
                                        .isOpen(isOpen)
                                        .isOptimize(BusinessConstant.ONE)
                                        .build())
                        );
                    } else {
                        //全关
                        wellheadViewOutput.setDurationList(
                                CollUtil.newArrayList(SpaceOptimizeDateDuration.builder()
                                        .openWellTime(BusinessConstant.START_OF_DAY_TIME)
                                        .closeWellTime(BusinessConstant.END_OF_DAY_TIME)
                                        .isOpen(BusinessConstant.ZERO)
                                        .isOptimize(BusinessConstant.ONE)
                                        .build())
                        );
                    }
                }
            }
            output.setWellheadList(wellheadViewOutputs);
        }
        return XSingleResult.success(output);
    }
}