package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.prediction.service.IStoragePredictedPowerCloudService;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerInput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerOutput;
import pps.core.prediction.entity.StoragePredictedPowerDataEnt;
import pps.core.prediction.mapper.StoragePredictedPowerDataMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 储能预测电量数据Cloud模块(模拟数据测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/09/20 15:44
 */
@XService
public class StoragePredictedPowerCloudServiceImpl implements IStoragePredictedPowerCloudService {

    /**
     * 储能预测Cloud模块--条件查询时段储能电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryStoragePredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryStoragePredictedPowerOutput> queryAverageEnergyStorageListByParam(XContext context, DynamicQueryStoragePredictedPowerInput input) {
        String storageId = input.getStorageId();
        List<String> storageIds = input.getStorageIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        StoragePredictedPowerDataMapper mapper = context.getBean(StoragePredictedPowerDataMapper.class);
        List<StoragePredictedPowerDataEnt> list = mapper.selectList(
                new QueryWrapper<StoragePredictedPowerDataEnt>()
                        .select("hour_time",
                                "min_time",
                                "AVG( power ) AS power")
                        .lambda()
                        .eq(StringUtils.isNotBlank(storageId), StoragePredictedPowerDataEnt::getStorageId, storageId)
                        .in(CollUtil.isNotEmpty(storageIds), StoragePredictedPowerDataEnt::getStorageId, storageIds)
                        .eq(StringUtils.isNotBlank(yearTime), StoragePredictedPowerDataEnt::getYearTime, yearTime)
                        .eq(StringUtils.isNotBlank(monthTime), StoragePredictedPowerDataEnt::getMonthTime, monthTime)
                        .between(!StringUtils.isAnyBlank(startTime, endTime), StoragePredictedPowerDataEnt::getDataDate, startTime, endTime)
                        .groupBy(StoragePredictedPowerDataEnt::getHourTime,
                                StoragePredictedPowerDataEnt::getMinTime)
                        .orderByAsc(StoragePredictedPowerDataEnt::getHourTime, StoragePredictedPowerDataEnt::getMinTime)
        );
        List<DynamicQueryStoragePredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryStoragePredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }
}
